package net.sf.itcb.addons.monitoring.track;

import net.sf.itcb.addons.monitoring.track.appender.ItcbAppender;
import net.sf.itcb.addons.monitoring.track.bean.ItcbTrackBean;

import org.aspectj.lang.ProceedingJoinPoint;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Required;

/**
 * This interceptor's aim is to log time spent in each layer and the result {@link TracerInterceptor#Status}.<br/>
 * It defines a transaction UUID if not already defined for the current thread.<br/>
 * This id will allow the monitoring visual tool to show the spent time in each layer.<br/>
 * <br/>
 * This interceptor has to be declared in Spring context file as in the example bellow :
 * <pre>{@code 
   	<aop:config>
		<aop:aspect id="aopItcbAddonTrackerInterceptorWithArgsOnTechnicalErrors"  ref="itcbAddonTrackerInterceptorWithArgsOnlyOnTechnicalErrors">
			    <aop:around
			      pointcut-ref="server" 
			      method="track"/>
		</aop:aspect>
		<aop:aspect id="aopItcbAddonTrackerInterceptorWithNoArgs"  ref="itcbAddonTrackerInterceptorWithNoArgsOnErrors">
			    <aop:around
			      pointcut-ref="page" 
			      method="track"/>
			    <aop:around
			      pointcut-ref="service" 
			      method="track"/>
			    <aop:around
			      pointcut-ref="dao" 
			      method="track"/>
		</aop:aspect>
	</aop:config>
 
 }</pre>
 <br/>
 * You can use and adapt this declaration in order to track specifically each layer. Available values for <code>ref</code> are :
 * <ul>
 * <li>itcbAddonTrackerInterceptorWithArgsOnAllErrors</li>
 * <li>itcbAddonTrackerInterceptorWithArgsOnlyOnTechnicalErrors</li>
 * <li>itcbAddonTrackerInterceptorWithArgsOnlyOnBusinessErrors</li>
 * <li>itcbAddonTrackerInterceptorWithNoArgsOnErrors</li>
 * </ul>
 * 
 * You just have to take care, not to use twice a pattern. Tracking would be done twice. So the list must contain only one interceptor.<br/>
 * 
 * You also can add your own pointcut. Basic pointcut are already declared for page, server, service (client and server side) and dao. <br/>
 * You can see it in itcb-monitoring-trace-applicationContext.xml in this package.<br/>
 * 
 * @author Pierre Le Roux
 * @since ITCB common 0.2.0
 * 
 */

public class TrackerInterceptor {
    
	

	private boolean trackParamsOnTechnicalError;
	private boolean trackParamsOnBusinessError;
	private ItcbAppender itcbAppender;
	
	public boolean isTrackParamsOnTechnicalError() {
		return trackParamsOnTechnicalError;
	}

	public void setTrackParamsOnTechnicalError(boolean trackParamsOnTechnicalError) {
		this.trackParamsOnTechnicalError = trackParamsOnTechnicalError;
	}

	public boolean isTrackParamsOnBusinessError() {
		return trackParamsOnBusinessError;
	}

	public void setTrackParamsOnBusinessError(boolean trackParamsOnBusinessError) {
		this.trackParamsOnBusinessError = trackParamsOnBusinessError;
	}
	
	@Required
	public void setItcbAppender(ItcbAppender itcbAppender) {
		this.itcbAppender = itcbAppender;
	}

	protected final Logger logger = LoggerFactory.getLogger(getClass());
	
	public Object track(ProceedingJoinPoint pjp) throws Throwable {
		 logger.debug("track");
		 long startTime = System.currentTimeMillis();
		  
		 ItcbTrackBean itcbTrackBean = new ItcbTrackBean();
		 itcbTrackBean.setClassName(pjp.getSignature().getDeclaringTypeName());
		 itcbTrackBean.setMethod(pjp.getSignature().getName());
		 itcbTrackBean.setStartTime(startTime);
		 itcbTrackBean.setTransactionId(TransactionIdHolder.getTransactionId());
		 try {
	    	Object ret=pjp.proceed();
	    	
	    	itcbTrackBean.setTimeInMillis(System.currentTimeMillis() - startTime);
	    	itcbAppender.track(itcbTrackBean);
	    	logger.debug("end track");
		    return ret;
	     }
	     catch(Throwable t) {
	    	itcbTrackBean.setTimeInMillis(System.currentTimeMillis() - startTime);
		    itcbTrackBean.setException(t);
		    itcbTrackBean.setParameters(pjp.getArgs());
		    itcbAppender.track(itcbTrackBean);
	    	throw t;
	     }

	 }	
	
}