package net.sf.itcb.addons.monitoring.track.appender;

import java.io.PrintWriter;
import java.io.StringWriter;
import java.io.Writer;
import java.sql.Connection;
import java.sql.PreparedStatement;

import javax.wsdl.Types;

import net.sf.itcb.addons.monitoring.track.bean.ItcbTrackBean;
import net.sf.itcb.common.business.core.ItcbConstants;
import net.sf.itcb.common.business.core.ItcbProjectInformation;
import net.sf.itcb.common.business.exceptions.BusinessItcbException;

import org.slf4j.MDC;
import org.springframework.jdbc.core.support.JdbcDaoSupport;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Repository;

/**
 * Simple DB appender that track in a spring configured database<br/>
 * Tracking in database is done in asynchronous thread, in order to minimize impact
 * 
 * @author Pierre Le Roux
 */
@Repository
public class ItcbDbAppender extends JdbcDaoSupport implements ItcbAppender {

	private static final String DONE = "DONE";
	private static final String TECHNICAL_ERROR = "TECHNICAL_ERROR";
	private static final String BUSINESS_ERROR = "BUSINESS_ERROR";
	
	private String module;
	private String moduleVersion;
		
	public void setItcbProjectInformation(
			ItcbProjectInformation itcbProjectInformation) {
		this.module = itcbProjectInformation.getGroupId() + "." + itcbProjectInformation.getArtifactId();
		this.moduleVersion = itcbProjectInformation.getVersion();
	}

	private static final String SQL_INSERT_TRACK =
		"INSERT INTO track (date, transactionId, user, module, moduleVersion, class, method, timeInMillis, status, exceptionType, exceptionMessage, requestMessage) " +
		"VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)";
	
	@Async
	public void track(ItcbTrackBean itcbTrackBean) {
		Connection con = null;
		logger.debug("track");
		try {
			con = getConnection();
			PreparedStatement stmt = con.prepareStatement(SQL_INSERT_TRACK);
			
			stmt.setLong(1, itcbTrackBean.getStartTime());
			stmt.setString(2, itcbTrackBean.getTransactionId());
			stmt.setString(3, MDC.get(ItcbConstants.MDC_USER_KEY));
			stmt.setString(4, this.module); 
			stmt.setString(5, this.moduleVersion); 
			stmt.setString(6, itcbTrackBean.getClassName());
			stmt.setString(7, itcbTrackBean.getMethod());
			stmt.setLong(8, itcbTrackBean.getTimeInMillis());
			Throwable t = itcbTrackBean.getException();
			if( t == null) {
				stmt.setString(9, DONE);
				stmt.setNull(10, Types.STRING_TYPE);
				stmt.setNull(11, Types.STRING_TYPE);
				stmt.setNull(12, Types.STRING_TYPE);
			}
			else  {
				if(t instanceof BusinessItcbException) {
					stmt.setString(9, BUSINESS_ERROR);
				}
				else {
					stmt.setString(9, TECHNICAL_ERROR);
				}
				stmt.setString(10, t.getClass().getName());
				stmt.setString(11, getStackTrace(t));
				stmt.setNull(12, Types.STRING_TYPE); // TODO Arguments set to null but has to inserted if asked
			}
				
			stmt.executeUpdate();

		} catch (Exception e) {
			logger.error("An unexpected exception has occurs in traking method : " + e.getMessage());
		} finally {
			releaseConnection(con);
		}
		
	}
	
	public static String getStackTrace(Throwable aThrowable) {
	    final Writer result = new StringWriter();
	    final PrintWriter printWriter = new PrintWriter(result);
	    aThrowable.printStackTrace(printWriter);
	    return result.toString();
	  }

}
