package net.sf.itcb.addons.monitoring.track.portlet;

import java.io.IOException;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.EventRequest;
import javax.portlet.EventResponse;
import javax.portlet.PortletException;
import javax.portlet.PortletRequest;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;
import javax.portlet.ResourceRequest;
import javax.portlet.ResourceResponse;
import javax.portlet.filter.ActionFilter;
import javax.portlet.filter.EventFilter;
import javax.portlet.filter.FilterChain;
import javax.portlet.filter.FilterConfig;
import javax.portlet.filter.RenderFilter;
import javax.portlet.filter.ResourceFilter;

import net.sf.itcb.addons.monitoring.track.TransactionIdHolder;
import net.sf.itcb.addons.monitoring.track.appender.ItcbAppender;
import net.sf.itcb.addons.monitoring.track.bean.ItcbTrackBean;
import net.sf.itcb.common.business.core.ItcbApplicationContextHolder;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * This filter generates a transactionId for all requests and store it in ThreadLocal.<br/>
 * This filter has to be declared in portlet.xml file as in the following example :<br/>
 * 
 * <code>
 *  &lt;filter><br/>
		&lt;filter-name>ItcbTrackerPortletFilter&lt;/filter-name><br/>
		&lt;filter-class>net.sf.itcb.addons.monitoring.portlet.TrackPortletFilter&lt;/filter-class><br/>
		&lt;lifecycle>ACTION_PHASE&lt;/lifecycle><br/>
		&lt;lifecycle>EVENT_PHASE&lt;/lifecycle><br/>
		&lt;lifecycle>RENDER_PHASE&lt;/lifecycle><br/>
		&lt;lifecycle>RESOURCE_PHASE&lt;/lifecycle><br/>
	&lt;/filter><br/>
	
	<br/>
	&lt;filter-mapping><br/>
		&lt;filter-name>ItcbTrackerPortletFilter&lt;/filter-name><br/>
		&lt;portlet-name><i>yourportletname</i>&lt;/portlet-name><br/>
	&lt;/filter-mapping><br/>
 * </code><br/>
 * 
 * You can customize lifecycle phase but it is highly recommended to keep ACTION, EVENT and RENDER phase at least.
 *  
 * @author Pierre Le Roux
 */


public class TrackerPortletFilter implements RenderFilter, ActionFilter, ResourceFilter, EventFilter {

	protected final Logger log = LoggerFactory.getLogger(getClass());
	
	private ItcbAppender itcbAppender;	
	
	@Override
	public void init(FilterConfig filterConfig) throws PortletException {
		log.info("TrackerPortletFilter initialized");
		this.itcbAppender = ItcbApplicationContextHolder.getContext().getBean("itcbTrackerAppender", ItcbAppender.class);
	}

	@Override
	public void destroy() {
		log.info("TrackerPortletFilter destroyed");
	}

	@Override
	public void doFilter(RenderRequest request, RenderResponse response,
			FilterChain chain) throws IOException, PortletException {
		long startTime = System.currentTimeMillis();
		try {
			chain.doFilter(request, response);
			doFilterDelegate(request, startTime, null);
		}
		catch(Throwable t) {
			doFilterDelegate(request, startTime, t);
		}
	}

	@Override
	public void doFilter(ResourceRequest request, ResourceResponse response,
			FilterChain chain) throws IOException, PortletException {
		long startTime = System.currentTimeMillis();
		try {
			chain.doFilter(request, response);
			doFilterDelegate(request, startTime, null);
		}
		catch(Throwable t) {
			doFilterDelegate(request, startTime, t);
		}
	}

	@Override
	public void doFilter(ActionRequest request, ActionResponse response,
			FilterChain chain) throws IOException, PortletException {
		long startTime = System.currentTimeMillis();
		try {
			chain.doFilter(request, response);
			doFilterDelegate(request, startTime, null);
		}
		catch(Throwable t) {
			doFilterDelegate(request, startTime, t);
		}
	}

	@Override
	public void doFilter(EventRequest request, EventResponse response,
			FilterChain chain) throws IOException, PortletException {
		long startTime = System.currentTimeMillis();
		try {
			chain.doFilter(request, response);
			doFilterDelegate(request, startTime, null);
		}
		catch(Throwable t) {
			doFilterDelegate(request, startTime, t);
		}
	}
	
	protected void doFilterDelegate(PortletRequest portletRequest, long startTime, Throwable t) throws IOException,
			PortletException {
		
		log.debug("track");
		  
		ItcbTrackBean itcbTrackBean = new ItcbTrackBean();
		itcbTrackBean.setClassName("net.sf.itcb.addons.monitoring.track.portlet.TrackerPortletFilter");

		itcbTrackBean.setStartTime(startTime);
		itcbTrackBean.setTransactionId(TransactionIdHolder.getTransactionId());
	    itcbTrackBean.setTimeInMillis(System.currentTimeMillis() - startTime);
		itcbTrackBean.setException(t);
		itcbTrackBean.setMethod(portletRequest.getParameter("toto"));
		itcbAppender.track(itcbTrackBean);
		
	}

}
