package net.sf.itcb.addons.monitoring.track.server;

import net.sf.itcb.addons.monitoring.track.Globals;
import net.sf.itcb.addons.monitoring.track.TransactionIdHolder;

import org.springframework.ws.context.MessageContext;
import org.springframework.ws.server.EndpointInterceptor;
import org.springframework.ws.soap.SoapHeader;
import org.springframework.ws.soap.SoapMessage;

/**
 * This server interceptor receives the transaction id from the client.<br/>
 * It retrieves it in SoapHeader with {@link Globals#HEADER_TRANSACTION_ID_QNAME} Qname.
 * 
 * This interceptor has to be activated in your server modules spring configuration as in the following example :<br/>
 * <code>
 *   &lt;bean class="org.springframework.ws.server.endpoint.mapping.PayloadRootAnnotationMethodEndpointMapping"&gt;<br/>
 *	    &lt;property name="interceptors"&gt;<br/>
 *		    &lt;list&gt;<br/>
 *            &lt;ref bean="trackerEndpointInterceptor"/&gt;<br/>
 *          ...<br/>
 *		      &lt;ref bean="validatingInterceptor" /&gt;<br/>
 *          ...<br/>
 *		      
 *		    &lt;/list&gt;<br/>
 *		&lt;/property&gt;<br/>
 *   &lt;/bean&gt;<br/><br/>
 *   
 *   &lt;bean id="trackerEndpointInterceptor" class="net.sf.itcb.addons.monitoring.track.server.TrackerEndpointInterceptor"/&gt;
 * </code> 
 * <br/>
 * This declaration has to be done in the webservice config file in your src/main/webapp/WEB-INF/xxxx-servlet.xml
 * @author Pierre Le Roux
 *
 */
public class TrackerEndpointInterceptor implements EndpointInterceptor {

	@Override
	public boolean handleFault(MessageContext messageContext, Object endpoint)
			throws Exception {
		return true;
	}

	@Override
	public boolean handleRequest(MessageContext messageContext, Object endpoint)
			throws Exception {
		SoapHeader soapHeader = ((SoapMessage) messageContext.getRequest()).getSoapHeader();
		if( soapHeader != null) {
			String transactionId = soapHeader.getAttributeValue(Globals.HEADER_TRANSACTION_ID_QNAME);
			
			// If transactionId has been set, we store it in the current Thread	
			if(transactionId!=null) {
				TransactionIdHolder.setTransactionId(transactionId);
			}
			else {
				// else we initialize it
				TransactionIdHolder.getTransactionId();
			}
		}
		return true;
	}

	@Override
	public boolean handleResponse(MessageContext messageContext, Object endpoint)
			throws Exception {
		return true;
	}

}
